import { Driver } from "../driver/Driver";
import { ConnectionOptions } from "./ConnectionOptions";
import { Repository } from "../repository/Repository";
import { EntityMetadata } from "../metadata/EntityMetadata";
import { ConstructorFunction } from "../common/ConstructorFunction";
import { EntityManager } from "../entity-manager/EntityManager";
import { ReactiveRepository } from "../repository/ReactiveRepository";
import { ReactiveEntityManager } from "../entity-manager/ReactiveEntityManager";
import { TreeRepository } from "../repository/TreeRepository";
import { ReactiveTreeRepository } from "../repository/ReactiveTreeRepository";
import { EntitySchema } from "../metadata/entity-schema/EntitySchema";
/**
 * A single connection instance to the database.
 * Each connection has its own entity manager, repositories, subscribers and metadatas.
 */
export declare class Connection {
    /**
     * The name of the connection.
     */
    readonly name: string;
    /**
     * Database driver used by this connection.
     */
    readonly driver: Driver;
    /**
     * Gets EntityManager of this connection.
     */
    private readonly _entityManager;
    /**
     * Gets ReactiveEntityManager of this connection.
     */
    private readonly _reactiveEntityManager;
    /**
     * Used to initialize lazy relations.
     */
    private readonly lazyRelationsWrapper;
    /**
     * All entity listener metadatas that are registered for this connection.
     */
    private readonly entityListeners;
    /**
     * All subscribers that are registered for this connection.
     */
    private readonly entitySubscribers;
    /**
     * Used to broadcast connection events.
     */
    private readonly broadcaster;
    /**
     * Connection options.
     */
    private readonly options;
    /**
     * All entity metadatas that are registered for this connection.
     */
    private readonly entityMetadatas;
    /**
     * Registered entity classes to be used for this connection.
     */
    private readonly entityClasses;
    /**
     * Registered entity schemas to be used for this connection.
     */
    private readonly entitySchemas;
    /**
     * Registered subscriber classes to be used for this connection.
     */
    private readonly subscriberClasses;
    /**
     * Registered naming strategy classes to be used for this connection.
     */
    private readonly namingStrategyClasses;
    /**
     * All connection's repositories.
     */
    private readonly repositories;
    /**
     * All connection's reactive repositories.
     */
    private readonly reactiveRepositories;
    /**
     * Naming strategy to be used in this connection.
     */
    private usedNamingStrategy;
    /**
     * Indicates if connection has been done or not.
     */
    private _isConnected;
    constructor(name: string, driver: Driver, options: ConnectionOptions);
    /**
     * Returns true if connection to the database already established for this connection.
     */
    readonly isConnected: boolean;
    /**
     * Entity manager allows to work with any entity of your connection.
     */
    readonly entityManager: EntityManager;
    /**
     * Entity manager allows to work with any entity of your connection.
     * This version of entity manager is reactive - works with Observables instead of promises.
     */
    readonly reactiveEntityManager: ReactiveEntityManager;
    /**
     * Performs connection to the database.
     */
    connect(): Promise<this>;
    /**
     * Closes this connection.
     */
    close(): Promise<void>;
    /**
     * Creates database schema for all entities registered in this connection.
     */
    syncSchema(dropBeforeSync?: boolean): Promise<void>;
    /**
     * Imports entities from the given paths (directories) for the current connection.
     */
    importEntitiesFromDirectories(paths: string[]): Promise<this>;
    /**
     * Imports entity schemas from the given paths (directories) for the current connection.
     */
    importEntitySchemaFromDirectories(paths: string[]): Promise<this>;
    /**
     * Imports subscribers from the given paths (directories) for the current connection.
     */
    importSubscribersFromDirectories(paths: string[]): Promise<this>;
    /**
     * Imports naming strategies from the given paths (directories) for the current connection.
     */
    importNamingStrategiesFromDirectories(paths: string[]): Promise<this>;
    /**
     * Imports entities for the current connection.
     */
    importEntities(entities: Function[]): Promise<this>;
    /**
     * Imports schemas for the current connection.
     */
    importEntitySchemas(schemas: EntitySchema[]): Promise<this>;
    /**
     * Imports entities for the given connection. If connection name is not given then default connection is used.
     */
    importSubscribers(subscriberClasses: Function[]): Promise<this>;
    /**
     * Imports entities for the current connection.
     */
    importNamingStrategies(strategies: Function[]): Promise<this>;
    /**
     * Sets given naming strategy to be used. Naming strategy must be set to be used before connection is established.
     */
    useNamingStrategy(name: string): this;
    /**
     * Sets given naming strategy to be used. Naming strategy must be set to be used before connection is established.
     */
    useNamingStrategy(strategy: Function): this;
    /**
     * Gets the entity metadata of the given entity target.
     */
    getMetadata(entity: Function): EntityMetadata;
    /**
     * Gets repository for the given entity class.
     */
    getRepository<Entity>(entityClass: ConstructorFunction<Entity> | Function): Repository<Entity>;
    /**
     * Gets repository for the given entity name.
     */
    getRepository<Entity>(entityClass: string): Repository<Entity>;
    /**
     * Gets repository for the given entity name.
     */
    getRepository<Entity>(entityClassOrName: ConstructorFunction<Entity> | Function | string): Repository<Entity>;
    /**
     * Gets tree repository for the given entity class.
     */
    getTreeRepository<Entity>(entityClass: ConstructorFunction<Entity> | Function): TreeRepository<Entity>;
    /**
     * Gets tree repository for the given entity name.
     */
    getTreeRepository<Entity>(entityName: string): TreeRepository<Entity>;
    /**
     * Gets tree repository for the given entity name.
     */
    getTreeRepository<Entity>(entityClassOrName: ConstructorFunction<Entity> | Function | string): Repository<Entity>;
    /**
     * Gets reactive repository for the given entity class.
     */
    getReactiveRepository<Entity>(entityClass: ConstructorFunction<Entity> | Function): ReactiveRepository<Entity>;
    /**
     * Gets reactive repository for the given entity name.
     */
    getReactiveRepository<Entity>(entityName: string): ReactiveRepository<Entity>;
    /**
     * Gets reactive repository for the given entity name.
     */
    getReactiveRepository<Entity>(entityClassOrName: ConstructorFunction<Entity> | Function | string): ReactiveRepository<Entity>;
    /**
     * Gets reactive tree repository for the given entity class.
     */
    getReactiveTreeRepository<Entity>(entityClass: ConstructorFunction<Entity> | Function): ReactiveTreeRepository<Entity>;
    /**
     * Gets reactive tree repository for the given entity name.
     */
    getReactiveTreeRepository<Entity>(entityName: string): ReactiveTreeRepository<Entity>;
    /**
     * Gets reactive tree repository for the given entity name.
     */
    getReactiveTreeRepository<Entity>(entityClassOrName: ConstructorFunction<Entity> | Function | string): ReactiveTreeRepository<Entity>;
    /**
     * Builds all registered metadatas.
     */
    private buildMetadatas();
    /**
     * Creates a naming strategy to be used for this connection.
     */
    private createNamingStrategy();
    /**
     * Creates repository and reactive repository for the given entity metadata.
     */
    private createRepository(metadata);
}
