"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const Repository_1 = require("../repository/Repository");
const RepositoryNotFoundError_1 = require("./error/RepositoryNotFoundError");
const EntityListenerMetadata_1 = require("../metadata/EntityListenerMetadata");
const DirectoryExportedClassesLoader_1 = require("../util/DirectoryExportedClassesLoader");
const index_1 = require("../index");
const EntityMetadataBuilder_1 = require("../metadata-builder/EntityMetadataBuilder");
const DefaultNamingStrategy_1 = require("../naming-strategy/DefaultNamingStrategy");
const EntityMetadataCollection_1 = require("../metadata-args/collection/EntityMetadataCollection");
const NoConnectionForRepositoryError_1 = require("./error/NoConnectionForRepositoryError");
const CannotImportAlreadyConnectedError_1 = require("./error/CannotImportAlreadyConnectedError");
const CannotCloseNotConnectedError_1 = require("./error/CannotCloseNotConnectedError");
const CannotConnectAlreadyConnectedError_1 = require("./error/CannotConnectAlreadyConnectedError");
const ReactiveRepository_1 = require("../repository/ReactiveRepository");
const NamingStrategyNotFoundError_1 = require("./error/NamingStrategyNotFoundError");
const EntityManagerFactory_1 = require("../entity-manager/EntityManagerFactory");
const RepositoryFactory_1 = require("../repository/RepositoryFactory");
const SchemaCreatorFactory_1 = require("../schema-creator/SchemaCreatorFactory");
const ReactiveRepositoryNotFoundError_1 = require("./error/ReactiveRepositoryNotFoundError");
const RepositoryNotTreeError_1 = require("./error/RepositoryNotTreeError");
const CannotSyncNotConnectedError_1 = require("./error/CannotSyncNotConnectedError");
const CannotUseNamingStrategyNotConnectedError_1 = require("./error/CannotUseNamingStrategyNotConnectedError");
const BroadcasterFactory_1 = require("../subscriber/BroadcasterFactory");
const CannotGetEntityManagerNotConnectedError_1 = require("./error/CannotGetEntityManagerNotConnectedError");
const LazyRelationsWrapper_1 = require("../repository/LazyRelationsWrapper");
/**
 * A single connection instance to the database.
 * Each connection has its own entity manager, repositories, subscribers and metadatas.
 */
class Connection {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    constructor(name, driver, options) {
        /**
         * All entity listener metadatas that are registered for this connection.
         */
        this.entityListeners = [];
        /**
         * All subscribers that are registered for this connection.
         */
        this.entitySubscribers = [];
        /**
         * All entity metadatas that are registered for this connection.
         */
        this.entityMetadatas = new EntityMetadataCollection_1.EntityMetadataCollection();
        /**
         * Registered entity classes to be used for this connection.
         */
        this.entityClasses = [];
        /**
         * Registered entity schemas to be used for this connection.
         */
        this.entitySchemas = [];
        /**
         * Registered subscriber classes to be used for this connection.
         */
        this.subscriberClasses = [];
        /**
         * Registered naming strategy classes to be used for this connection.
         */
        this.namingStrategyClasses = [];
        /**
         * All connection's repositories.
         */
        this.repositories = [];
        /**
         * All connection's reactive repositories.
         */
        this.reactiveRepositories = [];
        /**
         * Indicates if connection has been done or not.
         */
        this._isConnected = false;
        this.name = name;
        this.driver = driver;
        this.driver.connectionOptions = options;
        this.options = options;
        this._entityManager = index_1.getFromContainer(EntityManagerFactory_1.EntityManagerFactory).createEntityManager(this);
        this._reactiveEntityManager = index_1.getFromContainer(EntityManagerFactory_1.EntityManagerFactory).createReactiveEntityManager(this);
        this.broadcaster = index_1.getFromContainer(BroadcasterFactory_1.BroadcasterFactory).createBroadcaster(this.entityMetadatas, this.entitySubscribers, this.entityListeners);
        this.lazyRelationsWrapper = new LazyRelationsWrapper_1.LazyRelationsWrapper(this.driver, this.entityMetadatas, this.broadcaster);
    }
    // -------------------------------------------------------------------------
    // Accessors
    // -------------------------------------------------------------------------
    /**
     * Returns true if connection to the database already established for this connection.
     */
    get isConnected() {
        return this._isConnected;
    }
    /**
     * Entity manager allows to work with any entity of your connection.
     */
    get entityManager() {
        if (!this.isConnected)
            throw new CannotGetEntityManagerNotConnectedError_1.CannotGetEntityManagerNotConnectedError(this.name);
        return this._entityManager;
    }
    /**
     * Entity manager allows to work with any entity of your connection.
     * This version of entity manager is reactive - works with Observables instead of promises.
     */
    get reactiveEntityManager() {
        if (!this.isConnected)
            throw new CannotGetEntityManagerNotConnectedError_1.CannotGetEntityManagerNotConnectedError(this.name);
        return this._reactiveEntityManager;
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Performs connection to the database.
     */
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isConnected)
                throw new CannotConnectAlreadyConnectedError_1.CannotConnectAlreadyConnectedError(this.name);
            // connect to the database via its driver
            yield this.driver.connect();
            // build all metadatas registered in the current connection
            this.buildMetadatas();
            // set connected status for the current connection
            this._isConnected = true;
            // second build schema
            if (this.options.autoSchemaCreate === true)
                yield this.syncSchema();
            return this;
        });
    }
    /**
     * Closes this connection.
     */
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isConnected)
                throw new CannotCloseNotConnectedError_1.CannotCloseNotConnectedError(this.name);
            yield this.driver.disconnect();
            this._isConnected = false;
        });
    }
    /**
     * Creates database schema for all entities registered in this connection.
     */
    syncSchema(dropBeforeSync = false) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isConnected)
                throw new CannotSyncNotConnectedError_1.CannotSyncNotConnectedError(this.name);
            if (dropBeforeSync)
                yield this.driver.clearDatabase();
            const schemaBuilder = this.driver.createSchemaBuilder();
            const schemaCreatorFactory = index_1.getFromContainer(SchemaCreatorFactory_1.SchemaCreatorFactory);
            const schemaCreator = schemaCreatorFactory.create(schemaBuilder, this.entityMetadatas);
            return schemaCreator.create();
        });
    }
    /**
     * Imports entities from the given paths (directories) for the current connection.
     */
    importEntitiesFromDirectories(paths) {
        return __awaiter(this, void 0, void 0, function* () {
            this.importEntities(DirectoryExportedClassesLoader_1.importClassesFromDirectories(paths));
            return this;
        });
    }
    /**
     * Imports entity schemas from the given paths (directories) for the current connection.
     */
    importEntitySchemaFromDirectories(paths) {
        return __awaiter(this, void 0, void 0, function* () {
            this.importEntitySchemas(DirectoryExportedClassesLoader_1.importJsonsFromDirectories(paths));
            return this;
        });
    }
    /**
     * Imports subscribers from the given paths (directories) for the current connection.
     */
    importSubscribersFromDirectories(paths) {
        return __awaiter(this, void 0, void 0, function* () {
            this.importSubscribers(DirectoryExportedClassesLoader_1.importClassesFromDirectories(paths));
            return this;
        });
    }
    /**
     * Imports naming strategies from the given paths (directories) for the current connection.
     */
    importNamingStrategiesFromDirectories(paths) {
        return __awaiter(this, void 0, void 0, function* () {
            this.importEntities(DirectoryExportedClassesLoader_1.importClassesFromDirectories(paths));
            return this;
        });
    }
    /**
     * Imports entities for the current connection.
     */
    importEntities(entities) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isConnected)
                throw new CannotImportAlreadyConnectedError_1.CannotImportAlreadyConnectedError("entities", this.name);
            entities.forEach(cls => this.entityClasses.push(cls));
            return this;
        });
    }
    /**
     * Imports schemas for the current connection.
     */
    importEntitySchemas(schemas) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isConnected)
                throw new CannotImportAlreadyConnectedError_1.CannotImportAlreadyConnectedError("schemas", this.name);
            schemas.forEach(schema => this.entitySchemas.push(schema));
            return this;
        });
    }
    /**
     * Imports entities for the given connection. If connection name is not given then default connection is used.
     */
    importSubscribers(subscriberClasses) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isConnected)
                throw new CannotImportAlreadyConnectedError_1.CannotImportAlreadyConnectedError("entity subscribers", this.name);
            subscriberClasses.forEach(cls => this.subscriberClasses.push(cls));
            return this;
        });
    }
    /**
     * Imports entities for the current connection.
     */
    importNamingStrategies(strategies) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isConnected)
                throw new CannotImportAlreadyConnectedError_1.CannotImportAlreadyConnectedError("naming strategies", this.name);
            strategies.forEach(cls => this.namingStrategyClasses.push(cls));
            return this;
        });
    }
    /**
     * Sets given naming strategy to be used. Naming strategy must be set to be used before connection is established.
     */
    useNamingStrategy(strategyClassOrName) {
        if (this.isConnected)
            throw new CannotUseNamingStrategyNotConnectedError_1.CannotUseNamingStrategyNotConnectedError(this.name);
        this.usedNamingStrategy = strategyClassOrName;
        return this;
    }
    /**
     * Gets the entity metadata of the given entity target.
     */
    getMetadata(entity) {
        return this.entityMetadatas.findByTarget(entity);
    }
    /**
     * Gets repository for the given entity class or name.
     */
    getRepository(entityClassOrName) {
        if (!this.isConnected)
            throw new NoConnectionForRepositoryError_1.NoConnectionForRepositoryError(this.name);
        if (!this.entityMetadatas.hasTarget(entityClassOrName))
            throw new RepositoryNotFoundError_1.RepositoryNotFoundError(this.name, entityClassOrName);
        const metadata = this.entityMetadatas.findByTarget(entityClassOrName);
        const repository = this.repositories.find(repository => Repository_1.Repository.ownsMetadata(repository, metadata));
        if (!repository)
            throw new RepositoryNotFoundError_1.RepositoryNotFoundError(this.name, entityClassOrName);
        return repository;
    }
    /**
     * Gets tree repository for the given entity class or name.
     */
    getTreeRepository(entityClassOrName) {
        if (!this.isConnected)
            throw new NoConnectionForRepositoryError_1.NoConnectionForRepositoryError(this.name);
        if (!this.entityMetadatas.hasTarget(entityClassOrName))
            throw new RepositoryNotFoundError_1.RepositoryNotFoundError(this.name, entityClassOrName);
        const metadata = this.entityMetadatas.findByTarget(entityClassOrName);
        const repository = this.repositories.find(repository => Repository_1.Repository.ownsMetadata(repository, metadata));
        if (!repository)
            throw new RepositoryNotFoundError_1.RepositoryNotFoundError(this.name, entityClassOrName);
        if (!metadata.table.isClosure)
            throw new RepositoryNotTreeError_1.RepositoryNotTreeError(entityClassOrName);
        return repository;
    }
    /**
     * Gets reactive repository for the given entity class.
     */
    getReactiveRepository(entityClassOrName) {
        if (!this.isConnected)
            throw new NoConnectionForRepositoryError_1.NoConnectionForRepositoryError(this.name);
        if (!this.entityMetadatas.hasTarget(entityClassOrName))
            throw new RepositoryNotFoundError_1.RepositoryNotFoundError(this.name, entityClassOrName);
        const metadata = this.entityMetadatas.findByTarget(entityClassOrName);
        const repository = this.reactiveRepositories.find(repository => ReactiveRepository_1.ReactiveRepository.ownsMetadata(repository, metadata));
        if (!repository)
            throw new ReactiveRepositoryNotFoundError_1.ReactiveRepositoryNotFoundError(this.name, entityClassOrName);
        return repository;
    }
    /**
     * Gets reactive tree repository for the given entity class.
     */
    getReactiveTreeRepository(entityClassOrName) {
        if (!this.isConnected)
            throw new NoConnectionForRepositoryError_1.NoConnectionForRepositoryError(this.name);
        if (!this.entityMetadatas.hasTarget(entityClassOrName))
            throw new RepositoryNotFoundError_1.RepositoryNotFoundError(this.name, entityClassOrName);
        const metadata = this.entityMetadatas.findByTarget(entityClassOrName);
        const repository = this.reactiveRepositories.find(repository => ReactiveRepository_1.ReactiveRepository.ownsMetadata(repository, metadata));
        if (!repository)
            throw new RepositoryNotFoundError_1.RepositoryNotFoundError(this.name, entityClassOrName);
        if (!metadata.table.isClosure)
            throw new RepositoryNotTreeError_1.RepositoryNotTreeError(entityClassOrName);
        return repository;
    }
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    /**
     * Builds all registered metadatas.
     */
    buildMetadatas() {
        const namingStrategy = this.createNamingStrategy();
        // take imported event subscribers
        if (this.subscriberClasses && this.subscriberClasses.length) {
            index_1.getMetadataArgsStorage()
                .entitySubscribers
                .filterByTargets(this.subscriberClasses)
                .map(metadata => index_1.getFromContainer(metadata.target))
                .forEach(subscriber => this.entitySubscribers.push(subscriber));
        }
        // take imported entity listeners
        if (this.entityClasses && this.entityClasses.length) {
            index_1.getMetadataArgsStorage()
                .entityListeners
                .filterByTargets(this.entityClasses)
                .forEach(metadata => this.entityListeners.push(new EntityListenerMetadata_1.EntityListenerMetadata(metadata)));
        }
        // build entity metadatas from metadata args storage (collected from decorators)
        if (this.entityClasses && this.entityClasses.length) {
            index_1.getFromContainer(EntityMetadataBuilder_1.EntityMetadataBuilder)
                .buildFromMetadataArgsStorage(this.lazyRelationsWrapper, namingStrategy, this.entityClasses)
                .forEach(metadata => {
                this.entityMetadatas.push(metadata);
                this.createRepository(metadata);
            });
        }
        // build entity metadatas from given entity schemas
        if (this.entitySchemas && this.entitySchemas.length) {
            index_1.getFromContainer(EntityMetadataBuilder_1.EntityMetadataBuilder)
                .buildFromSchemas(this.lazyRelationsWrapper, namingStrategy, this.entitySchemas)
                .forEach(metadata => {
                this.entityMetadatas.push(metadata);
                this.createRepository(metadata);
            });
        }
    }
    /**
     * Creates a naming strategy to be used for this connection.
     */
    createNamingStrategy() {
        // if naming strategies are not loaded, or used naming strategy is not set then use default naming strategy
        if (!this.namingStrategyClasses || !this.namingStrategyClasses.length || !this.usedNamingStrategy)
            return index_1.getFromContainer(DefaultNamingStrategy_1.DefaultNamingStrategy);
        // try to find used naming strategy in the list of loaded naming strategies
        const namingMetadata = index_1.getMetadataArgsStorage()
            .namingStrategies
            .filterByTargets(this.namingStrategyClasses)
            .find(strategy => {
            if (typeof this.usedNamingStrategy === "string") {
                return strategy.name === this.usedNamingStrategy;
            }
            else {
                return strategy.target === this.usedNamingStrategy;
            }
        });
        // throw an error if not found
        if (!namingMetadata)
            throw new NamingStrategyNotFoundError_1.NamingStrategyNotFoundError(this.usedNamingStrategy, this.name);
        // initialize a naming strategy instance
        return index_1.getFromContainer(namingMetadata.target);
    }
    /**
     * Creates repository and reactive repository for the given entity metadata.
     */
    createRepository(metadata) {
        const repositoryFactory = index_1.getFromContainer(RepositoryFactory_1.RepositoryFactory);
        if (metadata.table.isClosure) {
            const repository = repositoryFactory.createTreeRepository(this, this.broadcaster, this.entityMetadatas, metadata);
            const reactiveRepository = repositoryFactory.createReactiveTreeRepository(repository);
            this.repositories.push(repository);
            this.reactiveRepositories.push(reactiveRepository);
        }
        else {
            const repository = repositoryFactory.createRepository(this, this.broadcaster, this.entityMetadatas, metadata);
            const reactiveRepository = repositoryFactory.createReactiveRepository(repository);
            this.repositories.push(repository);
            this.reactiveRepositories.push(reactiveRepository);
        }
    }
}
exports.Connection = Connection;
//# sourceMappingURL=Connection.js.map