"use strict";
const ConnectionIsNotSetError_1 = require("./error/ConnectionIsNotSetError");
const BaseDriver_1 = require("./BaseDriver");
const PostgresSchemaBuilder_1 = require("../schema-builder/PostgresSchemaBuilder");
/**
 * This driver organizes work with postgres database.
 */
class PostgresDriver extends BaseDriver_1.BaseDriver {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    constructor(postgres) {
        super();
        // if driver dependency is not given explicitly, then try to load it via "require"
        if (!postgres && require) {
            try {
                postgres = require("pg");
            }
            catch (e) {
                throw new Error("Postgres package has not been found installed. Try to install it: npm install pg --save");
            }
        }
        else {
            throw new Error("Cannot load postgres driver dependencies. Try to install all required dependencies.");
        }
        this.postgres = postgres;
    }
    // -------------------------------------------------------------------------
    // Getter Methods
    // -------------------------------------------------------------------------
    /**
     * Access to the native implementation of the database.
     */
    get native() {
        return this.postgres;
    }
    /**
     * Access to the native connection to the database.
     */
    get nativeConnection() {
        return this.postgresConnection;
    }
    /**
     * Database name to which this connection is made.
     */
    get db() {
        // if (this.postgresConnection && this.postgresConnection.config.database)
        //     return this.postgresConnection.config.database;
        if (this.connectionOptions.database)
            return this.connectionOptions.database;
        throw new Error("Cannot get the database name. Since database name is not explicitly given in configuration " +
            "(maybe connection url is used?), database name cannot be retrieved until connection is made.");
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Creates a schema builder which can be used to build database/table schemas.
     */
    createSchemaBuilder() {
        return new PostgresSchemaBuilder_1.PostgresSchemaBuilder(this);
    }
    /**
     * Performs connection to the database based on given connection options.
     */
    connect() {
        return new Promise((ok, fail) => {
            this.postgresConnection = new this.postgres.Client({
                host: this.connectionOptions.host,
                user: this.connectionOptions.username,
                password: this.connectionOptions.password,
                database: this.connectionOptions.database,
                port: this.connectionOptions.port
            });
            this.postgresConnection.connect((err) => err ? fail(err) : ok());
        });
    }
    /**
     * Closes connection with database.
     */
    disconnect() {
        this.checkIfConnectionSet();
        return new Promise((ok, fail) => {
            this.postgresConnection.end(); // todo: check if it can emit errors
            ok();
        });
    }
    /**
     * Escapes given value.
     */
    escape(value) {
        return this.postgresConnection.escape(value);
    }
    /**
     * Executes a given SQL query.
     */
    query(query) {
        this.checkIfConnectionSet();
        this.logQuery(query);
        return new Promise((ok, fail) => this.postgresConnection.query(query, (err, result) => {
            if (err) {
                this.logFailedQuery(query);
                this.logQueryError(err);
                fail(err);
            }
            else {
                ok(result.rows);
            }
        }));
    }
    /**
     * Clears all tables in the currently connected database.
     */
    clearDatabase() {
        this.checkIfConnectionSet(); // todo:
        const disableForeignKeysCheckQuery = `SET FOREIGN_KEY_CHECKS = 0;`;
        const dropTablesQuery = `SELECT concat('DROP TABLE IF EXISTS ', table_name, ';') AS q FROM ` +
            `information_schema.tables WHERE table_schema = '${this.db}';`;
        const enableForeignKeysCheckQuery = `SET FOREIGN_KEY_CHECKS = 1;`;
        return this.query(disableForeignKeysCheckQuery)
            .then(() => this.query(dropTablesQuery))
            .then(results => Promise.all(results.map(q => this.query(q["q"]))))
            .then(() => this.query(enableForeignKeysCheckQuery))
            .then(() => { });
    }
    // -------------------------------------------------------------------------
    // Protected Methods
    // -------------------------------------------------------------------------
    checkIfConnectionSet() {
        if (!this.postgresConnection)
            throw new ConnectionIsNotSetError_1.ConnectionIsNotSetError("postgres");
    }
}
exports.PostgresDriver = PostgresDriver;
//# sourceMappingURL=PostgresDriver.js.map