"use strict";
/**
 * Entity manager supposed to work with any entity, automatically find its repository and call its methods, whatever
 * entity type are you passing.
 */
class EntityManager {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    constructor(connection) {
        this.connection = connection;
    }
    /**
     * Gets repository for the given entity class or name.
     */
    getRepository(entityClassOrName) {
        if (typeof entityClassOrName === "string") {
            return this.connection.getRepository(entityClassOrName);
        }
        else {
            return this.connection.getRepository(entityClassOrName);
        }
    }
    /**
     * Gets repository for the given entity class or name.
     */
    getTreeRepository(entityClassOrName) {
        if (typeof entityClassOrName === "string") {
            return this.connection.getTreeRepository(entityClassOrName);
        }
        else {
            return this.connection.getTreeRepository(entityClassOrName);
        }
    }
    /**
     * Gets reactive repository of the given entity.
     */
    getReactiveRepository(entityClass) {
        return this.connection.getReactiveRepository(entityClass);
    }
    hasId(targetOrEntity, maybeEntity) {
        const target = arguments.length === 2 ? targetOrEntity : targetOrEntity.constructor;
        const entity = arguments.length === 2 ? maybeEntity : targetOrEntity;
        return this.getRepository(target).hasId(entity);
    }
    /**
     * Creates a new query builder that can be used to build an sql query.
     */
    createQueryBuilder(entityClass, alias) {
        return this.getRepository(entityClass).createQueryBuilder(alias);
    }
    /**
     * Creates a new entity. If fromRawEntity is given then it creates a new entity and copies all entity properties
     * from this object into a new entity (copies only properties that should be in a new entity).
     */
    create(entityClass, fromRawEntity) {
        return this.getRepository(entityClass).create(fromRawEntity);
    }
    /**
     * Creates a entities from the given array of plain javascript objects.
     */
    createMany(entityClass, copyFromObjects) {
        return this.getRepository(entityClass).createMany(copyFromObjects);
    }
    /**
     * Creates a new entity from the given plan javascript object. If entity already exist in the database, then
     * it loads it (and everything related to it), replaces all values with the new ones from the given object
     * and returns this new entity. This new entity is actually a loaded from the db entity with all properties
     * replaced from the new object.
     */
    initialize(entityClass, object) {
        return this.getRepository(entityClass).initialize(object);
    }
    /**
     * Merges two entities into one new entity.
     */
    merge(entityClass, ...objects) {
        return this.getRepository(entityClass).merge(...objects);
    }
    persist(targetOrEntity, maybeEntity) {
        // todo: extra casting is used strange tsc error here, check later maybe typescript bug
        const target = arguments.length === 2 ? targetOrEntity : targetOrEntity.constructor;
        const entity = arguments.length === 2 ? maybeEntity : targetOrEntity;
        return this.getRepository(target).persist(entity);
    }
    remove(targetOrEntity, maybeEntity) {
        // todo: extra casting is used strange tsc error here, check later maybe typescript bug
        const target = arguments.length === 2 ? targetOrEntity : targetOrEntity.constructor;
        const entity = arguments.length === 2 ? maybeEntity : targetOrEntity;
        return this.getRepository(target).remove(entity);
    }
    /**
     * Finds entities that match given conditions.
     */
    find(entityClass, conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return this.getRepository(entityClass).find(conditionsOrFindOptions, options);
        }
        else if (conditionsOrFindOptions) {
            return this.getRepository(entityClass).find(conditionsOrFindOptions);
        }
        else {
            return this.getRepository(entityClass).find();
        }
    }
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(entityClass, conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return this.getRepository(entityClass).findAndCount(conditionsOrFindOptions, options);
        }
        else if (conditionsOrFindOptions) {
            return this.getRepository(entityClass).findAndCount(conditionsOrFindOptions);
        }
        else {
            return this.getRepository(entityClass).findAndCount();
        }
    }
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(entityClass, conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return this.getRepository(entityClass).findOne(conditionsOrFindOptions, options);
        }
        else if (conditionsOrFindOptions) {
            return this.getRepository(entityClass).findOne(conditionsOrFindOptions);
        }
        else {
            return this.getRepository(entityClass).findOne();
        }
    }
    /**
     * Finds entity with given id.
     */
    findOneById(entityClass, id, options) {
        return this.getRepository(entityClass).findOneById(id, options);
    }
    /**
     * Executes raw SQL query and returns raw database results.
     */
    query(query) {
        return this.connection.driver.query(query);
    }
    /**
     * Wraps given function execution (and all operations made there) in a transaction.
     */
    transaction(runInTransaction) {
        let runInTransactionResult;
        return this.connection.driver
            .beginTransaction()
            .then(() => runInTransaction())
            .then(result => {
            runInTransactionResult = result;
            return this.connection.driver.commitTransaction();
        })
            .then(() => runInTransactionResult);
    }
    setRelation(entityClass, relationName, entityId, relatedEntityId) {
        return this.getRepository(entityClass).setRelation(relationName, entityId, relatedEntityId);
    }
    addToRelation(entityClass, relationName, entityId, relatedEntityIds) {
        return this.getRepository(entityClass).addToRelation(relationName, entityId, relatedEntityIds);
    }
    removeFromRelation(entityClass, relationName, entityId, relatedEntityIds) {
        return this.getRepository(entityClass).removeFromRelation(relationName, entityId, relatedEntityIds);
    }
    addAndRemoveFromRelation(entityClass, relation, entityId, addRelatedEntityIds, removeRelatedEntityIds) {
        return this.getRepository(entityClass).addAndRemoveFromRelation(relation, entityId, addRelatedEntityIds, removeRelatedEntityIds);
    }
    /**
     * Removes entity with the given id.
     * Note that event listeners and event subscribers won't work (and will not send any events) when using this operation.
     */
    removeById(entityClass, id) {
        return this.getRepository(entityClass).removeById(id);
    }
    /**
     * Removes all entities with the given ids.
     * Note that event listeners and event subscribers won't work (and will not send any events) when using this operation.
     */
    removeByIds(entityClass, ids) {
        return this.getRepository(entityClass).removeByIds(ids);
    }
    /**
     * Roots are entities that have no ancestors. Finds them all.
     */
    findRoots(entityClass) {
        return this.getTreeRepository(entityClass).findRoots();
    }
    /**
     * Creates a query builder used to get descendants of the entities in a tree.
     */
    createDescendantsQueryBuilder(entityClass, alias, closureTableAlias, entity) {
        return this.getTreeRepository(entityClass).createDescendantsQueryBuilder(alias, closureTableAlias, entity);
    }
    /**
     * Gets all children (descendants) of the given entity. Returns them all in a flat array.
     */
    findDescendants(entityClass, entity) {
        return this.getTreeRepository(entityClass).findDescendants(entity);
    }
    /**
     * Gets all children (descendants) of the given entity. Returns them in a tree - nested into each other.
     */
    findDescendantsTree(entityClass, entity) {
        return this.getTreeRepository(entityClass).findDescendantsTree(entity);
    }
    /**
     * Gets number of descendants of the entity.
     */
    countDescendants(entityClass, entity) {
        return this.getTreeRepository(entityClass).countDescendants(entity);
    }
    /**
     * Creates a query builder used to get ancestors of the entities in the tree.
     */
    createAncestorsQueryBuilder(entityClass, alias, closureTableAlias, entity) {
        return this.getTreeRepository(entityClass).createAncestorsQueryBuilder(alias, closureTableAlias, entity);
    }
    /**
     * Gets all parents (ancestors) of the given entity. Returns them all in a flat array.
     */
    findAncestors(entityClass, entity) {
        return this.getTreeRepository(entityClass).findAncestors(entity);
    }
    /**
     * Gets all parents (ancestors) of the given entity. Returns them in a tree - nested into each other.
     */
    findAncestorsTree(entityClass, entity) {
        return this.getTreeRepository(entityClass).findAncestorsTree(entity);
    }
    /**
     * Gets number of ancestors of the entity.
     */
    countAncestors(entityClass, entity) {
        return this.getTreeRepository(entityClass).countAncestors(entity);
    }
}
exports.EntityManager = EntityManager;
//# sourceMappingURL=EntityManager.js.map