"use strict";
/**
 * Executes PersistOperation in the given connection.
 */
class PersistOperationExecutor {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    constructor(driver, entityMetadatas, broadcaster) {
        this.driver = driver;
        this.entityMetadatas = entityMetadatas;
        this.broadcaster = broadcaster;
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Executes given persist operation.
     */
    executePersistOperation(persistOperation) {
        let isTransactionStartedByItself = false;
        return Promise.resolve()
            .then(() => this.broadcastBeforeEvents(persistOperation))
            .then(() => {
            if (!this.driver.isTransactionActive()) {
                isTransactionStartedByItself = true;
                return this.driver.beginTransaction();
            }
            return Promise.resolve();
        })
            .then(() => this.executeInsertOperations(persistOperation))
            .then(() => this.executeInsertClosureTableOperations(persistOperation))
            .then(() => this.executeUpdateTreeLevelOperations(persistOperation))
            .then(() => this.executeInsertJunctionsOperations(persistOperation))
            .then(() => this.executeRemoveJunctionsOperations(persistOperation))
            .then(() => this.executeRemoveRelationOperations(persistOperation))
            .then(() => this.executeUpdateRelationsOperations(persistOperation))
            .then(() => this.executeUpdateInverseRelationsOperations(persistOperation))
            .then(() => this.executeUpdateOperations(persistOperation))
            .then(() => this.executeRemoveOperations(persistOperation))
            .then(() => {
            if (isTransactionStartedByItself === true)
                return this.driver.commitTransaction();
            return Promise.resolve();
        })
            .then(() => this.updateIdsOfInsertedEntities(persistOperation))
            .then(() => this.updateIdsOfRemovedEntities(persistOperation))
            .then(() => this.updateSpecialColumnsInEntities(persistOperation))
            .then(() => this.broadcastAfterEvents(persistOperation))
            .catch(error => {
            if (isTransactionStartedByItself === true) {
                return this.driver.rollbackTransaction()
                    .then(() => {
                    throw error;
                })
                    .catch(() => {
                    throw error;
                });
            }
            throw error;
        });
    }
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    /**
     * Broadcast all before persistment events - beforeInsert, beforeUpdate and beforeRemove events.
     */
    broadcastBeforeEvents(persistOperation) {
        const insertEvents = persistOperation.inserts.map(insertOperation => {
            const persistedEntityWithId = persistOperation.allPersistedEntities.find(e => e.entity === insertOperation.entity);
            if (!persistedEntityWithId)
                throw new Error(`Persisted entity was not found`);
            return this.broadcaster.broadcastBeforeInsertEvent(persistedEntityWithId.entity);
        });
        const updateEvents = persistOperation.updates.map(updateOperation => {
            const persistedEntityWithId = persistOperation.allPersistedEntities.find(e => e.entity === updateOperation.entity);
            if (!persistedEntityWithId)
                throw new Error(`Persisted entity was not found`);
            return this.broadcaster.broadcastBeforeUpdateEvent(persistedEntityWithId.entity, updateOperation.columns);
        });
        const removeEvents = persistOperation.removes.map(removeOperation => {
            // we can send here only dbEntity, not entity from the persisted object, since entity from the persisted
            // object does not exist anymore - its removed, and there is no way to find this removed object
            return this.broadcaster.broadcastBeforeRemoveEvent(removeOperation.entity, removeOperation.entityId);
        });
        return Promise.all(insertEvents)
            .then(() => Promise.all(updateEvents))
            .then(() => Promise.all(removeEvents)); // todo: do we really should send it in order?
    }
    /**
     * Broadcast all after persistment events - afterInsert, afterUpdate and afterRemove events.
     */
    broadcastAfterEvents(persistOperation) {
        const insertEvents = persistOperation.inserts.map(insertOperation => {
            const persistedEntity = persistOperation.allPersistedEntities.find(e => e.entity === insertOperation.entity);
            if (!persistedEntity)
                throw new Error(`Persisted entity was not found`);
            return this.broadcaster.broadcastAfterInsertEvent(persistedEntity.entity);
        });
        const updateEvents = persistOperation.updates.map(updateOperation => {
            const persistedEntityWithId = persistOperation.allPersistedEntities.find(e => e.entity === updateOperation.entity);
            if (!persistedEntityWithId)
                throw new Error(`Persisted entity was not found`);
            return this.broadcaster.broadcastAfterUpdateEvent(persistedEntityWithId.entity, updateOperation.columns);
        });
        const removeEvents = persistOperation.removes.map(removeOperation => {
            // we can send here only dbEntity, not entity from the persisted object, since entity from the persisted
            // object does not exist anymore - its removed, and there is no way to find this removed object
            return this.broadcaster.broadcastAfterRemoveEvent(removeOperation.entity, removeOperation.entityId);
        });
        return Promise.all(insertEvents)
            .then(() => Promise.all(updateEvents))
            .then(() => Promise.all(removeEvents)); // todo: do we really should send it in order?
    }
    /**
     * Executes insert operations.
     */
    executeInsertOperations(persistOperation) {
        return Promise.all(persistOperation.inserts.map(operation => {
            return this.insert(operation).then((insertId) => {
                operation.entityId = insertId;
            });
        }));
    }
    /**
     * Executes insert operations for closure tables.
     */
    executeInsertClosureTableOperations(persistOperation) {
        const promises = persistOperation.inserts
            .filter(operation => {
            const metadata = this.entityMetadatas.findByTarget(operation.target);
            return metadata.table.isClosure;
        })
            .map(operation => {
            const relationsUpdateMap = this.findUpdateOperationForEntity(persistOperation.updatesByRelations, persistOperation.inserts, operation.entity);
            return this.insertIntoClosureTable(operation, relationsUpdateMap).then(level => {
                operation.treeLevel = level;
            });
        });
        return Promise.all(promises);
    }
    /**
     * Executes update tree level operations in inserted entities right after data into closure table inserted.
     */
    executeUpdateTreeLevelOperations(persistOperation) {
        return Promise.all(persistOperation.inserts.map(operation => {
            return this.updateTreeLevel(operation);
        }));
    }
    /**
     * Executes insert junction operations.
     */
    executeInsertJunctionsOperations(persistOperation) {
        return Promise.all(persistOperation.junctionInserts.map(junctionOperation => {
            return this.insertJunctions(junctionOperation, persistOperation.inserts);
        }));
    }
    /**
     * Executes remove junction operations.
     */
    executeRemoveJunctionsOperations(persistOperation) {
        return Promise.all(persistOperation.junctionRemoves.map(junctionOperation => {
            return this.removeJunctions(junctionOperation);
        }));
    }
    /**
     * Executes update relations operations.
     */
    executeUpdateRelationsOperations(persistOperation) {
        return Promise.all(persistOperation.updatesByRelations.map(updateByRelation => {
            return this.updateByRelation(updateByRelation, persistOperation.inserts);
        }));
    }
    /**
     * Executes update relations operations.
     */
    executeUpdateInverseRelationsOperations(persistOperation) {
        return Promise.all(persistOperation.updatesByInverseRelations.map(updateInverseOperation => {
            return this.updateInverseRelation(updateInverseOperation, persistOperation.inserts);
        }));
    }
    /**
     * Executes update operations.
     */
    executeUpdateOperations(persistOperation) {
        return Promise.all(persistOperation.updates.map(updateOperation => {
            return this.update(updateOperation);
        }));
    }
    /**
     * Executes remove relations operations.
     */
    executeRemoveRelationOperations(persistOperation) {
        return Promise.all(persistOperation.removes
            .filter(operation => {
            return !!(operation.relation && !operation.relation.isManyToMany && !operation.relation.isOneToMany);
        })
            .map(operation => {
            return this.updateDeletedRelations(operation);
        }));
    }
    /**
     * Executes remove operations.
     */
    executeRemoveOperations(persistOperation) {
        return Promise.all(persistOperation.removes.map(operation => {
            return this.delete(operation.target, operation.entity);
        }));
    }
    /**
     * Updates all ids of the inserted entities.
     */
    updateIdsOfInsertedEntities(persistOperation) {
        persistOperation.inserts.forEach(insertOperation => {
            const metadata = this.entityMetadatas.findByTarget(insertOperation.target);
            insertOperation.entity[metadata.primaryColumn.propertyName] = insertOperation.entityId;
        });
    }
    /**
     * Updates all special columns of the saving entities (create date, update date, versioning).
     */
    updateSpecialColumnsInEntities(persistOperation) {
        persistOperation.inserts.forEach(insertOperation => {
            const metadata = this.entityMetadatas.findByTarget(insertOperation.target);
            if (metadata.hasUpdateDateColumn)
                insertOperation.entity[metadata.updateDateColumn.propertyName] = insertOperation.date;
            if (metadata.hasCreateDateColumn)
                insertOperation.entity[metadata.createDateColumn.propertyName] = insertOperation.date;
            if (metadata.hasVersionColumn)
                insertOperation.entity[metadata.versionColumn.propertyName]++;
            if (metadata.hasTreeLevelColumn) {
                // const parentEntity = insertOperation.entity[metadata.treeParentMetadata.propertyName];
                // const parentLevel = parentEntity ? (parentEntity[metadata.treeLevelColumn.propertyName] || 0) : 0;
                insertOperation.entity[metadata.treeLevelColumn.propertyName] = insertOperation.treeLevel;
            }
            /*if (metadata.hasTreeChildrenCountColumn) {
                insertOperation.entity[metadata.treeChildrenCountColumn.propertyName] = 0;
            }*/
        });
        persistOperation.updates.forEach(updateOperation => {
            const metadata = this.entityMetadatas.findByTarget(updateOperation.target);
            if (metadata.hasUpdateDateColumn)
                updateOperation.entity[metadata.updateDateColumn.propertyName] = updateOperation.date;
            if (metadata.hasCreateDateColumn)
                updateOperation.entity[metadata.createDateColumn.propertyName] = updateOperation.date;
            if (metadata.hasVersionColumn)
                updateOperation.entity[metadata.versionColumn.propertyName]++;
        });
    }
    /**
     * Removes all ids of the removed entities.
     */
    updateIdsOfRemovedEntities(persistOperation) {
        persistOperation.removes.forEach(removeOperation => {
            const metadata = this.entityMetadatas.findByTarget(removeOperation.target);
            const removedEntity = persistOperation.allPersistedEntities.find(allNewEntity => {
                return allNewEntity.entityTarget === removeOperation.target && allNewEntity.id === removeOperation.entity[metadata.primaryColumn.propertyName];
            });
            if (removedEntity)
                removedEntity.entity[metadata.primaryColumn.propertyName] = undefined;
        });
    }
    findUpdateOperationForEntity(operations, insertOperations, target) {
        let updateMap = {};
        operations
            .forEach(operation => {
            const relatedInsertOperation = insertOperations.find(o => o.entity === operation.targetEntity);
            const idInInserts = relatedInsertOperation ? relatedInsertOperation.entityId : null;
            if (operation.updatedRelation.isOneToMany) {
                const metadata = this.entityMetadatas.findByTarget(operation.insertOperation.target);
                if (operation.insertOperation.entity === target)
                    updateMap[operation.updatedRelation.inverseRelation.propertyName] = operation.targetEntity[metadata.primaryColumn.propertyName] || idInInserts;
            }
            else {
                if (operation.targetEntity === target)
                    updateMap[operation.updatedRelation.propertyName] = operation.insertOperation.entityId;
            }
        });
        return updateMap;
    }
    updateByRelation(operation, insertOperations) {
        let tableName, relationName, relationId, idColumn, id;
        const relatedInsertOperation = insertOperations.find(o => o.entity === operation.targetEntity);
        const idInInserts = relatedInsertOperation ? relatedInsertOperation.entityId : null;
        if (operation.updatedRelation.isOneToMany || operation.updatedRelation.isOneToOneNotOwner) {
            const metadata = this.entityMetadatas.findByTarget(operation.insertOperation.target);
            tableName = metadata.table.name;
            relationName = operation.updatedRelation.inverseRelation.name;
            relationId = operation.targetEntity[metadata.primaryColumn.propertyName] || idInInserts;
            idColumn = metadata.primaryColumn.name;
            id = operation.insertOperation.entityId;
        }
        else {
            const metadata = this.entityMetadatas.findByTarget(operation.entityTarget);
            tableName = metadata.table.name;
            relationName = operation.updatedRelation.name;
            relationId = operation.insertOperation.entityId;
            idColumn = metadata.primaryColumn.name;
            id = operation.targetEntity[metadata.primaryColumn.propertyName] || idInInserts;
        }
        return this.driver.update(tableName, { [relationName]: relationId }, { [idColumn]: id });
    }
    updateInverseRelation(operation, insertOperations) {
        const targetEntityMetadata = this.entityMetadatas.findByTarget(operation.entityTarget);
        const fromEntityMetadata = this.entityMetadatas.findByTarget(operation.fromEntityTarget);
        const tableName = targetEntityMetadata.table.name;
        const targetRelation = operation.fromRelation.inverseRelation;
        const idColumn = targetEntityMetadata.primaryColumn.name;
        const id = targetEntityMetadata.getEntityId(operation.targetEntity);
        const fromEntityInsertOperation = insertOperations.find(o => o.entity === operation.fromEntity);
        let targetEntityId; // todo: better do it during insertion - pass UpdateByInverseSideOperation[] to insert and do it there
        if (operation.operationType === "remove") {
            targetEntityId = null;
        }
        else {
            if (fromEntityInsertOperation && targetRelation.joinColumn.referencedColumn === fromEntityMetadata.primaryColumn) {
                targetEntityId = fromEntityInsertOperation.entityId;
            }
            else {
                targetEntityId = operation.fromEntity[targetRelation.joinColumn.referencedColumn.name];
            }
        }
        return this.driver.update(tableName, { [targetRelation.name]: targetEntityId }, { [idColumn]: id });
    }
    update(updateOperation) {
        const entity = updateOperation.entity;
        const metadata = this.entityMetadatas.findByTarget(updateOperation.target);
        const values = {};
        updateOperation.columns.forEach(column => {
            values[column.name] = this.driver.preparePersistentValue(column.getEntityValue(entity), column);
        });
        updateOperation.relations.forEach(relation => {
            const value = this.getEntityRelationValue(relation, entity);
            values[relation.name] = value !== null && value !== undefined ? value[relation.inverseEntityMetadata.primaryColumn.propertyName] : null;
        });
        // if number of updated columns = 0 no need to update updated date and version columns
        if (Object.keys(values).length === 0)
            return Promise.resolve();
        if (metadata.hasUpdateDateColumn)
            values[metadata.updateDateColumn.name] = this.driver.preparePersistentValue(new Date(), metadata.updateDateColumn);
        if (metadata.hasVersionColumn)
            values[metadata.versionColumn.name] = this.driver.preparePersistentValue(entity[metadata.versionColumn.propertyName] + 1, metadata.versionColumn);
        return this.driver.update(metadata.table.name, values, { [metadata.primaryColumn.name]: metadata.getEntityId(entity) });
    }
    updateDeletedRelations(removeOperation) {
        if (removeOperation.relation) {
            return this.driver.update(removeOperation.fromMetadata.table.name, { [removeOperation.relation.name]: null }, { [removeOperation.fromMetadata.primaryColumn.name]: removeOperation.fromEntityId });
        }
        throw new Error("Remove operation relation is not set"); // todo: find out how its possible
    }
    delete(target, entity) {
        const metadata = this.entityMetadatas.findByTarget(target);
        return this.driver.delete(metadata.table.name, { [metadata.primaryColumn.name]: entity[metadata.primaryColumn.propertyName] });
    }
    insert(operation) {
        const entity = operation.entity;
        const metadata = this.entityMetadatas.findByTarget(operation.target);
        const columns = metadata.columns
            .filter(column => !column.isVirtual && column.hasEntityValue(entity));
        const columnNames = columns.map(column => column.name);
        const values = columns.map(column => this.driver.preparePersistentValue(column.getEntityValue(entity), column));
        const relationColumns = metadata.relations
            .filter(relation => !relation.isManyToMany && relation.isOwning && !!relation.inverseEntityMetadata)
            .filter(relation => entity.hasOwnProperty(relation.propertyName))
            .map(relation => relation.name);
        const relationValues = metadata.relations
            .filter(relation => !relation.isManyToMany && relation.isOwning && !!relation.inverseEntityMetadata)
            .filter(relation => entity.hasOwnProperty(relation.propertyName))
            .map(relation => {
            const value = this.getEntityRelationValue(relation, entity);
            if (value !== null && value !== undefined)
                return value[relation.inverseEntityMetadata.primaryColumn.propertyName];
            return value;
        });
        const allColumns = columnNames.concat(relationColumns);
        const allValues = values.concat(relationValues);
        if (metadata.hasCreateDateColumn) {
            allColumns.push(metadata.createDateColumn.name);
            allValues.push(this.driver.preparePersistentValue(operation.date, metadata.createDateColumn));
        }
        if (metadata.hasUpdateDateColumn) {
            allColumns.push(metadata.updateDateColumn.name);
            allValues.push(this.driver.preparePersistentValue(operation.date, metadata.updateDateColumn));
        }
        if (metadata.hasVersionColumn) {
            allColumns.push(metadata.versionColumn.name);
            allValues.push(this.driver.preparePersistentValue(1, metadata.versionColumn));
        }
        if (metadata.hasTreeLevelColumn && metadata.hasTreeParentRelation) {
            const parentEntity = entity[metadata.treeParentRelation.name]; // todo: are you sure here we should use name and not propertyName ?
            const parentLevel = parentEntity ? (parentEntity[metadata.treeLevelColumn.propertyName] || 0) : 0;
            allColumns.push(metadata.treeLevelColumn.name);
            allValues.push(parentLevel + 1);
        }
        /*if (metadata.hasTreeChildrenCountColumn) {
            allColumns.push(metadata.treeChildrenCountColumn.name);
            allValues.push(0);
        }*/
        // console.log("inserting: ", this.zipObject(allColumns, allValues));
        return this.driver.insert(metadata.table.name, this.zipObject(allColumns, allValues));
    }
    insertIntoClosureTable(operation, updateMap) {
        const entity = operation.entity;
        const metadata = this.entityMetadatas.findByTarget(operation.target);
        const parentEntity = entity[metadata.treeParentRelation.propertyName];
        let parentEntityId = 0;
        if (parentEntity && parentEntity[metadata.primaryColumn.propertyName]) {
            parentEntityId = parentEntity[metadata.primaryColumn.propertyName];
        }
        else if (updateMap && updateMap[metadata.treeParentRelation.propertyName]) {
            parentEntityId = updateMap[metadata.treeParentRelation.propertyName];
        }
        return this.driver.insertIntoClosureTable(metadata.closureJunctionTable.table.name, operation.entityId, parentEntityId, metadata.hasTreeLevelColumn);
    }
    updateTreeLevel(operation) {
        const metadata = this.entityMetadatas.findByTarget(operation.target);
        if (metadata.hasTreeLevelColumn && operation.treeLevel) {
            const values = { [metadata.treeLevelColumn.name]: operation.treeLevel };
            return this.driver.update(metadata.table.name, values, { [metadata.primaryColumn.name]: operation.entityId });
        }
        return Promise.resolve();
    }
    insertJunctions(junctionOperation, insertOperations) {
        const junctionMetadata = junctionOperation.metadata;
        const metadata1 = this.entityMetadatas.findByTarget(junctionOperation.entity1Target);
        const metadata2 = this.entityMetadatas.findByTarget(junctionOperation.entity2Target);
        const columns = junctionMetadata.columns.map(column => column.name);
        const insertOperation1 = insertOperations.find(o => o.entity === junctionOperation.entity1);
        const insertOperation2 = insertOperations.find(o => o.entity === junctionOperation.entity2);
        let id1 = junctionOperation.entity1[metadata1.primaryColumn.propertyName];
        let id2 = junctionOperation.entity2[metadata2.primaryColumn.propertyName];
        if (!id1) {
            if (insertOperation1) {
                id1 = insertOperation1.entityId;
            }
            else {
                throw new Error(`Insert operation for ${junctionOperation.entity1} was not found.`);
            }
        }
        if (!id2) {
            if (insertOperation2) {
                id2 = insertOperation2.entityId;
            }
            else {
                throw new Error(`Insert operation for ${junctionOperation.entity2} was not found.`);
            }
        }
        let values;
        // order may differ, find solution (column.table to compare with entity metadata table?)
        if (metadata1.table === junctionMetadata.foreignKeys[0].referencedTable) {
            values = [id1, id2];
        }
        else {
            values = [id2, id1];
        }
        return this.driver.insert(junctionMetadata.table.name, this.zipObject(columns, values));
    }
    removeJunctions(junctionOperation) {
        const junctionMetadata = junctionOperation.metadata;
        const metadata1 = this.entityMetadatas.findByTarget(junctionOperation.entity1Target);
        const metadata2 = this.entityMetadatas.findByTarget(junctionOperation.entity2Target);
        const columns = junctionMetadata.columns.map(column => column.name);
        const id1 = junctionOperation.entity1[metadata1.primaryColumn.propertyName];
        const id2 = junctionOperation.entity2[metadata2.primaryColumn.propertyName];
        return this.driver.delete(junctionMetadata.table.name, { [columns[0]]: id1, [columns[1]]: id2 });
    }
    zipObject(keys, values) {
        return keys.reduce((object, column, index) => {
            object[column] = values[index];
            return object;
        }, {});
    }
    getEntityRelationValue(relation, entity) {
        return relation.isLazy ? entity["__" + relation.propertyName + "__"] : entity[relation.propertyName];
    }
}
exports.PersistOperationExecutor = PersistOperationExecutor;
//# sourceMappingURL=PersistOperationExecutor.js.map