import { QueryBuilder } from "../query-builder/QueryBuilder";
import { FindOptions } from "./FindOptions";
import { Repository } from "./Repository";
import * as Rx from "rxjs/Rx";
import { EntityMetadata } from "../metadata/EntityMetadata";
import { ObjectLiteral } from "../common/ObjectLiteral";
/**
 * Repository is supposed to work with your entity objects. Find entities, insert, update, delete, etc.
 * This version of Repository is using rxjs library and Observables instead of promises.
 */
export declare class ReactiveRepository<Entity> {
    protected repository: Repository<Entity>;
    constructor(repository: Repository<Entity>);
    /**
     * Repository's target is an object (or unique string in the case if entity is loaded from a schema) that is managed
     * by this repository.
     */
    readonly target: Function | string;
    /**
     * Checks if entity has an id.
     */
    hasId(entity: Entity): boolean;
    /**
     * Creates a new query builder that can be used to build a sql query.
     */
    createQueryBuilder(alias: string): QueryBuilder<Entity>;
    /**
     * Creates a new entity. If fromRawEntity is given then it creates a new entity and copies all entity properties
     * from this object into a new entity (copies only properties that should be in a new entity).
     */
    create(fromRawEntity?: Object): Entity;
    /**
     * Creates entities from a given array of plain javascript objects.
     */
    createMany(copyFromObjects: Object[]): Entity[];
    /**
     * Creates a new entity from the given plan javascript object. If entity already exist in the database, then
     * it loads it (and everything related to it), replaces all values with the new ones from the given object
     * and returns this new entity. This new entity is actually a loaded from the db entity with all properties
     * replaced from the new object.
     */
    initialize(object: Object): Rx.Observable<Entity>;
    /**
     * Merges two entities into one new entity.
     */
    merge(...objects: ObjectLiteral[]): Entity;
    /**
     * Persists (saves) a given entity in the database. If entity does not exist in the database then it inserts it,
     * else if entity already exist in the database then it updates it.
     */
    persist(entity: Entity): Rx.Observable<Entity>;
    /**
     * Removes a given entity from the database.
     */
    remove(entity: Entity): Rx.Observable<Entity>;
    /**
     * Finds all entities.
     */
    find(): Rx.Observable<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    find(conditions: Object): Rx.Observable<Entity[]>;
    /**
     * Finds entities that match given find options.
     */
    find(options: FindOptions): Rx.Observable<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    find(conditions: Object, options: FindOptions): Rx.Observable<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(): Rx.Observable<[Entity[], number]>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(conditions: Object): Rx.Observable<[Entity[], number]>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(options: FindOptions): Rx.Observable<[Entity[], number]>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(conditions: Object, options: FindOptions): Rx.Observable<[Entity[], number]>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(): Rx.Observable<Entity>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(conditions: Object): Rx.Observable<Entity>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(options: FindOptions): Rx.Observable<Entity>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(conditions: Object, options: FindOptions): Rx.Observable<Entity>;
    /**
     * Finds entity with given id.
     */
    findOneById(id: any, options?: FindOptions): Rx.Observable<Entity>;
    /**
     * Executes raw SQL query and returns raw database results.
     */
    query(query: string): Rx.Observable<any>;
    /**
     * Wraps given function execution (and all operations made there) in a transaction.
     */
    transaction(runInTransaction: () => Promise<any>): Rx.Observable<any>;
    /**
     * Sets given relatedEntityId to the value of the relation of the entity with entityId id.
     * Should be used when you want quickly and efficiently set a relation (for many-to-one and one-to-many) to some entity.
     * Note that event listeners and event subscribers won't work (and will not send any events) when using this operation.
     */
    setRelation(relationName: string, entityId: any, relatedEntityId: any): Rx.Observable<void>;
    setRelation(relationName: ((t: Entity) => string | any), entityId: any, relatedEntityId: any): Rx.Observable<void>;
    /**
     * Adds a new relation between two entities into relation's many-to-many table.
     * Should be used when you want quickly and efficiently add a relation between two entities.
     * Note that event listeners and event subscribers won't work (and will not send any events) when using this operation.
     */
    addToRelation(relationName: string, entityId: any, relatedEntityIds: any[]): Rx.Observable<void>;
    addToRelation(relationName: ((t: Entity) => string | any), entityId: any, relatedEntityIds: any[]): Rx.Observable<void>;
    /**
     * Removes a relation between two entities from relation's many-to-many table.
     * Should be used when you want quickly and efficiently remove a many-to-many relation between two entities.
     * Note that event listeners and event subscribers won't work (and will not send any events) when using this operation.
     */
    removeFromRelation(relationName: string, entityId: any, relatedEntityIds: any[]): Rx.Observable<void>;
    removeFromRelation(relationName: ((t: Entity) => string | any), entityId: any, relatedEntityIds: any[]): Rx.Observable<void>;
    /**
     * Performs both #addToRelation and #removeFromRelation operations.
     * Should be used when you want quickly and efficiently and and remove a many-to-many relation between two entities.
     * Note that event listeners and event subscribers won't work (and will not send any events) when using this operation.
     */
    addAndRemoveFromRelation(relation: string, entityId: any, addRelatedEntityIds: any[], removeRelatedEntityIds: any[]): Rx.Observable<void>;
    addAndRemoveFromRelation(relation: ((t: Entity) => string | any), entityId: any, addRelatedEntityIds: any[], removeRelatedEntityIds: any[]): Rx.Observable<void>;
    /**
     * Removes entity with the given id.
     * Note that event listeners and event subscribers won't work (and will not send any events) when using this operation.
     */
    removeById(id: any): Rx.Observable<void>;
    /**
     * Removes all entities with the given ids.
     * Note that event listeners and event subscribers won't work (and will not send any events) when using this operation.
     */
    removeByIds(ids: any[]): Rx.Observable<void>;
    static ownsMetadata(reactiveRepository: ReactiveRepository<any>, metadata: EntityMetadata): boolean;
}
