"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
/**
 * Creates indexes based on the given metadata.
 *
 * @internal
 */
class SchemaCreator {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    constructor(schemaBuilder, entityMetadatas) {
        this.schemaBuilder = schemaBuilder;
        this.entityMetadatas = entityMetadatas;
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Creates complete schemas for the given entity metadatas.
     */
    create() {
        return __awaiter(this, void 0, void 0, function* () {
            const metadatas = this.entityMetadatas;
            yield this.dropForeignKeysForAll(metadatas);
            yield this.createTablesForAll(metadatas);
            yield this.updateOldColumnsForAll(metadatas);
            yield this.dropRemovedColumnsForAll(metadatas);
            yield this.addNewColumnsForAll(metadatas);
            yield this.updateExistColumnsForAll(metadatas);
            yield this.createForeignKeysForAll(metadatas);
            yield this.updateUniqueKeysForAll(metadatas);
            yield this.createIndicesForAll(metadatas);
            yield this.removePrimaryKeyForAll(metadatas);
        });
    }
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    dropForeignKeysForAll(metadatas) {
        return Promise.all(metadatas.map(metadata => this.dropForeignKeys(metadata.table, metadata.foreignKeys)));
    }
    createTablesForAll(metadatas) {
        return Promise.all(metadatas.map(metadata => this.createNewTable(metadata.table, metadata.columns)));
    }
    updateOldColumnsForAll(metadatas) {
        const allKeys = metadatas.reduce((all, metadata) => all.concat(metadata.foreignKeys), []);
        return Promise.all(metadatas.map(metadata => this.updateOldColumns(metadata.table, metadata.columns, allKeys)));
    }
    dropRemovedColumnsForAll(metadatas) {
        const allKeys = metadatas.reduce((all, metadata) => all.concat(metadata.foreignKeys), []);
        return Promise.all(metadatas.map(metadata => this.dropRemovedColumns(metadata.table, metadata.columns, allKeys)));
    }
    addNewColumnsForAll(metadatas) {
        return Promise.all(metadatas.map(metadata => this.addNewColumns(metadata.table, metadata.columns)));
    }
    updateExistColumnsForAll(metadatas) {
        const allKeys = metadatas.reduce((all, metadata) => all.concat(metadata.foreignKeys), []);
        return Promise.all(metadatas.map(metadata => this.updateExistColumns(metadata.table, metadata.columns, allKeys)));
    }
    createForeignKeysForAll(metadatas) {
        return Promise.all(metadatas.map(metadata => this.createForeignKeys(metadata.table, metadata.foreignKeys)));
    }
    updateUniqueKeysForAll(metadatas) {
        return Promise.all(metadatas.map(metadata => this.updateUniqueKeys(metadata.table, metadata.columns)));
    }
    createIndicesForAll(metadatas) {
        return Promise.all(metadatas.map(metadata => this.createIndices(metadata.table, metadata.indices)));
    }
    removePrimaryKeyForAll(metadatas) {
        const queries = metadatas
            .filter(metadata => !metadata.hasPrimaryColumn)
            .map(metadata => this.removePrimaryKey(metadata.table));
        return Promise.all(queries);
    }
    /**
     * Drops all (old) foreign keys that exist in the table, but does not exist in the metadata.
     */
    dropForeignKeys(table, foreignKeys) {
        return this.schemaBuilder.getTableForeignQuery(table.name).then(dbKeys => {
            const dropKeysQueries = dbKeys
                .filter(dbKey => !foreignKeys.find(foreignKey => foreignKey.name === dbKey))
                .map(dbKey => this.schemaBuilder.dropForeignKeyQuery(table.name, dbKey));
            return Promise.all(dropKeysQueries);
        });
    }
    /**
     * Creates a new table if it does not exist.
     */
    createNewTable(table, columns) {
        return this.schemaBuilder.checkIfTableExist(table.name).then(exist => {
            if (!exist)
                return this.schemaBuilder.createTableQuery(table, columns);
            return undefined;
        });
    }
    /**
     * Renames (and updates) all columns that has "oldColumnName".
     */
    updateOldColumns(table, columns, foreignKeys) {
        return __awaiter(this, void 0, void 0, function* () {
            const dbColumns = yield this.schemaBuilder.getTableColumns(table.name);
            const updates = columns
                .filter(column => !!column.oldColumnName && column.name !== column.oldColumnName)
                .filter(column => dbColumns.indexOf(column.oldColumnName) !== -1)
                .map((column) => __awaiter(this, void 0, void 0, function* () {
                yield this.dropColumnForeignKeys(table.name, column.name, foreignKeys);
                return this.schemaBuilder.changeColumnQuery(table.name, column.oldColumnName, column);
            }));
            yield Promise.all(updates);
        });
    }
    /**
     * Drops all columns exist (left old) in the table, but does not exist in the metadata.
     */
    dropRemovedColumns(table, columns, foreignKeys) {
        return __awaiter(this, void 0, void 0, function* () {
            const dbColumns = yield this.schemaBuilder.getTableColumns(table.name);
            const dropColumnQueries = dbColumns
                .filter(dbColumn => !columns.find(column => column.name === dbColumn))
                .map((dbColumn) => __awaiter(this, void 0, void 0, function* () {
                yield this.dropColumnForeignKeys(table.name, dbColumn, foreignKeys);
                return this.schemaBuilder.dropColumnQuery(table.name, dbColumn);
            }));
            return Promise.all(dropColumnQueries);
        });
    }
    /**
     * Adds columns from metadata which does not exist in the table.
     */
    addNewColumns(table, columns) {
        return this.schemaBuilder.getTableColumns(table.name).then(dbColumns => {
            const newColumnQueries = columns
                .filter(column => dbColumns.indexOf(column.name) === -1)
                .map(column => this.schemaBuilder.addColumnQuery(table.name, column));
            return Promise.all(newColumnQueries);
        });
    }
    /**
     * Update all exist columns which metadata has changed.
     */
    updateExistColumns(table, columns, foreignKeys) {
        return __awaiter(this, void 0, void 0, function* () {
            const changedColumns = yield this.schemaBuilder.getChangedColumns(table.name, columns);
            const updateQueries = changedColumns.map((changedColumn) => __awaiter(this, void 0, void 0, function* () {
                const column = columns.find(column => column.name === changedColumn.columnName);
                if (!column)
                    throw new Error(`Column ${changedColumn.columnName} was not found in the given columns`);
                yield this.dropColumnForeignKeys(table.name, column.name, foreignKeys);
                return this.schemaBuilder.changeColumnQuery(table.name, column.name, column, changedColumn.hasPrimaryKey);
            }));
            return Promise.all(updateQueries);
        });
    }
    /**
     * Creates foreign keys which does not exist in the table yet.
     */
    createForeignKeys(table, foreignKeys) {
        return this.schemaBuilder.getTableForeignQuery(table.name).then(dbKeys => {
            const dropKeysQueries = foreignKeys
                .filter(foreignKey => dbKeys.indexOf(foreignKey.name) === -1)
                .map(foreignKey => this.schemaBuilder.addForeignKeyQuery(foreignKey));
            return Promise.all(dropKeysQueries);
        });
    }
    /**
     * Creates unique keys which are missing in db yet, and drops unique keys which exist in the db,
     * but does not exist in the metadata anymore.
     */
    updateUniqueKeys(table, columns) {
        return this.schemaBuilder.getTableUniqueKeysQuery(table.name).then(dbKeys => {
            // first find metadata columns that should be unique and update them if they are not unique in db
            const addQueries = columns
                .filter(column => column.isUnique)
                .filter(column => dbKeys.indexOf("uk_" + column.name) === -1)
                .map(column => this.schemaBuilder.addUniqueKey(table.name, column.name, "uk_" + column.name));
            // second find columns in db that are unique, however in metadata columns they are not unique
            const dropQueries = columns
                .filter(column => !column.isUnique)
                .filter(column => dbKeys.indexOf("uk_" + column.name) !== -1)
                .map(column => this.schemaBuilder.dropIndex(table.name, "uk_" + column.name));
            return Promise.all([addQueries, dropQueries]);
        });
    }
    /**
     * Creates indices which are missing in db yet, and drops indices which exist in the db,
     * but does not exist in the metadata anymore.
     */
    createIndices(table, compositeIndices) {
        return this.schemaBuilder.getTableIndicesQuery(table.name).then(tableIndices => {
            // drop all indices that exist in the table, but does not exist in the given composite indices
            const dropQueries = tableIndices
                .filter(tableIndex => !compositeIndices.find(compositeIndex => compositeIndex.name === tableIndex.key))
                .map(tableIndex => this.schemaBuilder.dropIndex(table.name, tableIndex.key));
            // then create table indices for all composite indices we have
            const addQueries = compositeIndices
                .filter(compositeIndex => !tableIndices.find(i => i.key === compositeIndex.name))
                .map(compositeIndex => this.schemaBuilder.createIndex(table.name, compositeIndex));
            return Promise.all([dropQueries, addQueries]);
        });
    }
    /**
     * Removes primary key from the table (if it was before and does not exist in the metadata anymore).
     */
    removePrimaryKey(table) {
        // find primary key name in db and remove it, because we don't have (anymore) primary key in the metadata
        return this.schemaBuilder.getPrimaryConstraintName(table.name).then(constraintName => {
            if (constraintName)
                return this.schemaBuilder.dropIndex(table.name, constraintName);
            return undefined;
        });
    }
    dropColumnForeignKeys(tableName, columnName, foreignKeys) {
        return __awaiter(this, void 0, void 0, function* () {
            const dependForeignKeys = foreignKeys.filter(foreignKey => {
                if (foreignKey.tableName === tableName) {
                    return !!foreignKey.columns.find(fkColumn => {
                        return fkColumn.name === columnName;
                    });
                }
                else if (foreignKey.referencedTableName === tableName) {
                    return !!foreignKey.referencedColumns.find(fkColumn => {
                        return fkColumn.name === columnName;
                    });
                }
                return false;
            });
            if (dependForeignKeys && dependForeignKeys.length) {
                yield Promise.all(dependForeignKeys.map((fk) => __awaiter(this, void 0, void 0, function* () {
                    const tableForeignKeys = yield this.schemaBuilder.getTableForeignQuery(fk.tableName);
                    if (!!tableForeignKeys.find(tableForeignKey => tableForeignKey === fk.name)) {
                        return this.schemaBuilder.dropForeignKeyQuery(fk.tableName, fk.name);
                    }
                })));
            }
        });
    }
}
exports.SchemaCreator = SchemaCreator;
//# sourceMappingURL=SchemaCreator.js.map